#include <stdlib.h>
#include <gtk/gtk.h>

int num_buttons = 5;
GtkWidget ** buttons;
GtkWidget * hscale;

gint button_toggle(GtkWidget *widget, GdkEvent *event, gpointer data)
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == FALSE)
    {
        return TRUE;
    }
    
    int current = 0;
    for (int i = 0; i < num_buttons; i++)
    {
        if (buttons[i] != widget)
        {
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[i]), FALSE);
        }
        else
        {
            current = i;
        }
    }
    
    gtk_range_set_value(GTK_RANGE(hscale), current + 1);

    return TRUE;
}

void scale_changed()
{
    int button_index = gtk_range_get_value(GTK_RANGE(hscale)) - 1;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[button_index]), TRUE);
}

static void destroy(GtkWidget *widget, gpointer data)
{
    gtk_main_quit();
}

int main(int argc, char *argv[])
{
    gtk_init(&argc,&argv);
    
    if (argc != 2)
    {
        printf("Usage: checkslider [buttons_count]\n");
        return 0;
    }
    
    num_buttons = atoi(argv[1]);
    if (num_buttons < 1 || num_buttons > 20)
    {
        printf("Invalid button count\n");
        return 0;
    }
   
    GtkWidget * window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    g_signal_connect(G_OBJECT(window), "destroy",G_CALLBACK(destroy),NULL);

    GtkWidget * vbox = gtk_vbox_new(TRUE,10);

    buttons = malloc(sizeof(GtkWidget *) * num_buttons);

    for (int i = 0; i < num_buttons; i++)
    {
        char caption[512] = {0};
        sprintf(caption, "Checkbutton #%d", i + 1);
        
        buttons[i] = gtk_check_button_new_with_label(caption);
        
        g_signal_connect(G_OBJECT(buttons[i]), "toggled", G_CALLBACK(button_toggle), NULL);
        gtk_box_pack_start(GTK_BOX(vbox), buttons[i], TRUE, TRUE, 4);
        gtk_widget_show(buttons[i]);
    }
    
    hscale = gtk_hscale_new_with_range(1, num_buttons, 1);
    g_signal_connect(G_OBJECT(hscale), "value-changed", G_CALLBACK(scale_changed), NULL);
    
    gtk_box_pack_start(GTK_BOX(vbox), hscale, TRUE, TRUE, 4);
    gtk_widget_show(hscale);
    
    gtk_container_add(GTK_CONTAINER(window),vbox);
    gtk_widget_show(vbox);
    gtk_widget_show(window);
    
    gtk_main();
}

